/* Written by Yong Li, Depeng Liu                                       */
/* Copyright (c) 2016                  	                               */
/* This program is free software: you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
 /* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License    */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

package cn.ac.ios.automata;


import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import Jama.Matrix;
import cn.ac.ios.automata.words.Alphabet;
import cn.ac.ios.automata.words.ContextWord;
import cn.ac.ios.graph.EdgeProperty;
import cn.ac.ios.graph.GraphSimple;
import cn.ac.ios.graph.ModelType;
import cn.ac.ios.graph.Properties;
import cn.ac.ios.value.ContextValue;
import cn.ac.ios.value.Type;
import cn.ac.ios.value.Value;

public class MultiplicityAutomaton implements Automaton {
	
	public class Tuple{
		public Tuple(int to, double w) {
			state = to;
			weight = w;
		}
		int state ;
		double weight ;
	}

	private final ContextValue contextValue;
	private GraphSimple graph;
	private final ContextWord contextWord;
	private final BitSet acceptingStates = new BitSet();
	private int currentState = -1 ;
	private Map<Integer, List<Tuple>> transMap = new HashMap<>();
	private List<Double> StatesWeights = new ArrayList<>();
	
	public MultiplicityAutomaton (ContextWord contextWord) {
		assert contextWord != null;
		this.contextWord = contextWord;
		this.contextValue = contextWord.getContextValue();
		graph = new GraphSimple(contextValue);
		
		Type typeModel = contextValue.newTypeObject(ModelType.class);
		Value valueModel = graph.addSettableGraphProperty(Properties.SEMANTICS, typeModel);
		valueModel.set(ModelType.WEIGHT);
		Type typeInteger = contextValue.newTypeInteger();
		graph.addSettableEdgeProperty(Properties.EXPRESSION, typeInteger);
		Type typeDouble = contextValue.newTypeDouble();
		graph.addSettableEdgeProperty(Properties.WEIGHT, typeDouble);
	}
	@Override
	public ModelType getModelType() {
		return ModelType.WEIGHT;
	}

	@Override
	public Alphabet getAlphabet() {
		return contextWord.getAlphabet();
	}

	@Override
	public int getNumStates() {
		return graph.getNumNodes();
	}

	@Override
	public BitSet getInitialStates() {
		return graph.getInitialNodes();
	}

	@Override
	public BitSet getSuccessors(int state, int letter) {
		graph.queryNode(state);
		EdgeProperty edgeProp = graph.getEdgeProperty(Properties.EXPRESSION);
		BitSet successors = new BitSet(getNumStates());
		for(int succNr = 0; succNr < graph.getNumSuccessors(); succNr ++) {
			Value valueLetter = edgeProp.get(succNr);
			if(valueLetter.getInt() == letter) successors.set(graph.getSuccessorNode(succNr));
		}
		return successors;
	}
	
	@Override
	public List<Double> getWeights(int state, int letter){
		EdgeProperty edgeProp = graph.getEdgeProperty(Properties.EXPRESSION);
		EdgeProperty edgeWeight = graph.getEdgeProperty(Properties.WEIGHT);
		List<Double> weights = new ArrayList<>();
		for(int succNr = 0; succNr < graph.getNumSuccessors(); succNr ++) {
			Value valueLetter = edgeProp.get(succNr);
			if(valueLetter.getInt() == letter) 
				weights.add(edgeWeight.get(succNr).getDouble());
		}
		return weights;
	}

	@Override
	public boolean isAccepted(int state) {
		return acceptingStates.get(state);
	}

	@Override
	public void addNewState(int number) {
		currentState = number;
		graph.queryNode(currentState);
		transMap.clear();

	}

	@Override
	public void addTransition(int letter, int to) {
	}

	@Override
	public void addTransition(int letter, BitSet tos) {
	}
	
	public void addTransition(int letter, int to, double weight) {
		if (weight != 0){
			if (transMap.containsKey(letter))
				{DecimalFormat decimal = new DecimalFormat("#.####");
				weight = Double.parseDouble(decimal.format(weight));
				transMap.get(letter).add(new Tuple(to, weight));}
			else {
				DecimalFormat decimal = new DecimalFormat("#.####");
				weight = Double.parseDouble(decimal.format(weight));
				List<Tuple> newArray = new ArrayList<>();
				newArray.add(new Tuple(to, weight));
				transMap.put(letter, newArray);}
		}
	}
	

	

	@Override
	public void setAccepting() {
		acceptingStates.set(currentState);	

	}

	@Override
	public void setInitial() {
		BitSet inits = graph.getInitialNodes();
		inits.set(currentState);

	}
	public String toString() {
		return AutomataExporterDOT.toString(this);
	}
	
	@Override
	public void addNewStateEnd() {
		int num = 0;
		for(int i = 0; i < transMap.size(); i++){
			num += transMap.get(i).size();
		}
		graph.prepareNode(num);
		int succNr = 0;
		Type typeInteger = contextValue.newTypeInteger();
		Value valueInteger = typeInteger.newValue();
		EdgeProperty edgeProp = graph.getEdgeProperty(Properties.EXPRESSION);
		Type typeDouble = contextValue.newTypeDouble();
		Value valueDouble = typeDouble.newValue();
		EdgeProperty weight = graph.getEdgeProperty(Properties.WEIGHT);
		for(Map.Entry<Integer, List<Tuple>> entry : transMap.entrySet()) {
			List<Tuple> array = entry.getValue();
			for(Tuple element: array){
			graph.setSuccessorNode(succNr, element.state);
			valueInteger.set(entry.getKey());
			edgeProp.set(valueInteger, succNr);
			valueDouble.set(element.weight);
			weight.set(valueDouble, succNr);
			++ succNr;}
		}
	}
	@Override
	public void setStatesWeight(Matrix A, int size){
		StatesWeights.clear();
		for (int i = 0; i < size; i++)
			StatesWeights.add(A.get(i, 0));
	}
	
	public List<Double> getStatesWeight(){
		return StatesWeights;
	}
	
	@Override
	public BitSet getAcceptingStates() {
		return (BitSet) acceptingStates.clone();
	}
	
	
}
