/* Written by Yong Li, Depeng Liu                                       */
/* Copyright (c) 2016                  	                               */
/* This program is free software: you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
 /* the Free Software Foundation, either version 3 of the License, or    */
/* (at your option) any later version.                                  */

/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */

/* You should have received a copy of the GNU General Public License    */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

package cn.ac.ios.roll;

import java.util.ArrayList;
import java.util.List;

import automata.FiniteAutomaton;
import cn.ac.ios.automata.Automaton;
import cn.ac.ios.learner.Learner;
import cn.ac.ios.learner.fdfa.LearnerFDFA;
import cn.ac.ios.learner.omega.buechi.LearnerOmegaBuechi;
import cn.ac.ios.options.Options;
import cn.ac.ios.oracle.Minimizer;
import cn.ac.ios.util.AutomatonPrinter;

public class Statistics {
	
	public static void reset() {
		
//		numStatesTarget = 0;
//		numTransTarget = 0;
		
//		numStatesHypothesis = 0;
//		numTransHypothesis = 0;
		
		numMembershipQuery = 0;
		numEquivalenceQuery = 0;
		
		numStatesLeading = 0;
		numStatesProgress = null;
		
		timeMembershipQuery = 0;
		timeEquivalenceQuery = 0;
		timeTranslator = 0;
		timeLearner = 0;
		timeTotal = 0;
		
		numSamplingOmegaWord = 0;
		
	}
	
//	public static int numStatesTarget ;  // number of states in target automaton
//	public static int numTransTarget ;   // number of transitions in target automaton
	
//	public static int numStatesHypothesis ; // number of states in hypothesis
//	public static int numTransHypothesis  ; // number of transitions in hypothesis
	
	public static int numMembershipQuery ; // number of membership query
	public static int numEquivalenceQuery ; // number of equivalence query
	
	public static int numStatesLeading; // number of states in leading automaton
	public static List<Integer> numStatesProgress; // number of states in progress automata
	
	
	public static long timeMembershipQuery ; // milliseconds used in membership query 
	public static long timeEquivalenceQuery ;// milliseconds used in equivalence query
	public static long timeLastEquivalenceQuery; // time for last eq check
	public static long timeTotal; // milliseconds used in learning
	public static long timeTranslator; // milliseconds used in CE translation
	public static long timeLearner; // milliseconds used in FDFA learner
	
	public static long timeLearnerLeading; // milliseconds used in FDFA learner for leading automaton
	public static long timeLearnerProgress; // milliseconds used in FDFA learner for progress automata
	
	public static long timeMinimizationBuechi; // milliseconds used in Buechi minimization before eq check
	public static long timeSampling; //milliseconds used in sampling
	public static int numMembershipSampling; // number of membership queries in sampling
	public static int numSamplingSucceeded; // number of sampling succeeded
	public static int numSamplingTried; // number of sampling we tried
	
	// sampling as the teacher
	public static long numSamplingOmegaWord;
	
	// store automaton information
	public static FiniteAutomaton targetAut; // target automaton
	public static FiniteAutomaton hypothesis; // final hypothesis
	public static FiniteAutomaton ldba ; // LDBA output
	
	public static void print() {

		System.out.println("#T.S = " + targetAut.states.size() + "    //#states of target");
		System.out.println("#T.T = " + targetAut.trans + "    //#transitions of target");
		
		System.out.println("#H.S = " + hypothesis.states.size() + "    //#states of learned automaton");
		System.out.println("#H.T = " + hypothesis.trans + "    //#transitions of learned automaton");
		

		int numTotal = numStatesLeading;
		System.out.println("#L.S = " + numStatesLeading + "    // #states of leading automaton or L$");
		System.out.print("#P.S = [" );
		for(Integer numStates : numStatesProgress) {
			System.out.print(numStates + ", ");
			numTotal += numStates;
		}
		System.out.println(" ]    // #states of each progress automaton");
		// total number of the states in final FDFA
		System.out.println("#F.S = " + numTotal + "    // #L.S + #P.S");
		
		if(Options.outLDBA && ldba != null) {
			System.out.println("#LDBA.S = " + ldba.states.size() + "    // #states of LDBA");
			System.out.println("#LDBA.T = " + ldba.trans + "     // #transitions of LDBA");
		}
		
		System.out.println("#MQ = " + numMembershipQuery + "    // #membership query");
		System.out.println("#EQ = " + numEquivalenceQuery + "    // #equivalence query");
		
		if(Options.teacherSampling) {
			System.out.println("#ST.W = " + numSamplingOmegaWord);
		}
		
		System.out.println("#TMQ = " + timeMembershipQuery + " (ms)" + "    // time for membership queries");
		System.out.println("#TEQ = " + timeEquivalenceQuery + " (ms)" + "    // time for equivalence queries");
		System.out.println("#TLEQ = " + timeLastEquivalenceQuery + " (ms)" + "    // time for the last equivalence query");
		System.out.println("#TTR = " + timeTranslator + " (ms)" + "    // time for the translator");
		
		System.out.println("#TLR = " + timeLearner + " (ms)" + "    // time for the learner");
		System.out.println("#TLRL = " + timeLearnerLeading + " (ms)"  + "    // time for the learning leading automaton" );
		System.out.println("#TLRP = " + timeLearnerProgress + " (ms)" + "    // time for the learning progress automata" );
		
		if(Options.sampleFdfa) {
			System.out.println("#MQ.S = " + numMembershipSampling);
			System.out.println("#SA.T = " + numSamplingTried);
			System.out.println("#SA.S = " + numSamplingSucceeded);
			System.out.println("#TSA = " + timeSampling);
		}
		
		System.out.println("#TMB = " + timeMinimizationBuechi + " (ms)" + "    // time for minimizing Buechi automata");
		
		System.out.println("#TTO = " + timeTotal + " (ms)" + "    //total time for learning Buechi automata");
		
		
		System.out.println("target automaton: ");
		AutomatonPrinter.print(targetAut, System.out);
			
		System.out.println("final automaton learned: ");
		AutomatonPrinter.print(hypothesis, System.out);
//		System.out.println("minimized automaton: ");
//		FiniteAutomaton minAut = Minimizer.minimizeBuechi(hypothesis, 12);
//		System.out.println("#MH.S = " + minAut.states.size());
//		System.out.println("#MH.T = " + minAut.trans);
//		AutomatonPrinter.print(minAut, System.out);
		
		if(Options.outLDBA && ldba != null) {
			System.out.println("final LDBA learned: ");
			AutomatonPrinter.print(ldba, System.out);
		}
		
	}
	
	public static void setLearnerFDFA(Learner<Automaton, Boolean> learner) {
		numStatesProgress = new ArrayList<>();
		if(learner instanceof LearnerFDFA) {
			LearnerFDFA learnerFDFA = (LearnerFDFA)learner;
			numStatesLeading = learnerFDFA.getLeadingAutomaton().getNumStates();
			for(int autNr = 0; autNr < learnerFDFA.getNumProgressAutomata(); autNr ++) {
				numStatesProgress.add(learnerFDFA.getProgressAutomaton(autNr).getNumStates());
			}
		}else {
			LearnerOmegaBuechi learnerBuechi = (LearnerOmegaBuechi)learner;
			numStatesLeading = learnerBuechi.getLDollarLearner().getHypothesis().getNumStates();
		}
	}
	

}
